#! /bin/bash

########################################
#  usage: sh mcd.sh <TIME> <PATH> [1]  #
########################################
#  Danny Sateler González - 2521028-K  #
#  Sergio Silva Muñoz     - 2521025-5  #
########################################

#############################
#Verificación de argumentos.#
#############################
if [ $# -lt 2 ]; then
	echo "Cantidad de argumentos insuficiente. El modo de uso es:"
	echo "./mcd.sh <segundos> <directorio> [1]"
	exit 1
	else if [ $# -eq 2 ]; then
		mode=1
	else if [ $3 -eq 1 ];then
		mode=2
	else
		echo "Último argumento inválido. El modo de uso es:"
		echo "./mcd.sh <segundos> <directorio> [1]"
		exit 1
fi
fi
fi

#########################################
#Preparación de los archivos temporales.#
#########################################
rm -f $dir1 $dir2 $dir3
touch ./.1dir.txt
touch ./.2dir.txt
touch ./.res.txt
touch ./.log.txt
dir=$2
dir1=./.1dir.txt
dir2=./.2dir.txt
dir3=./.res.txt
dir4=./.log.txt

##############################################
#Setea los parámetros para el envío de correo#
##############################################
set_mail()
{
	#Set SMTP
	read -p "Ingrese el servidor SMTP a utilizar: " smtp
	#Servidor SMTP de gMail: smtp.gmail.com

	#Set Port
	read -p "Ingrese el puerto donde éste escuha: " port
	#Puertos de escucha SMTP de gMail: 587 o 465 

	#Set user mail
	read -p "Ingrese la cuenta email desde donde se enviará la alerta: " umail

	case $mode in
	#Modo 1 implementa la solución tal cual se pide en la tarea, entregando el pass en Base64
	1)
	read -p "Ingrese la clave de usuario de su cuenta en base64: " clave64
	pass=`perl -MMIME::Base64 -e 'print decode_base64("'$clave64'")'`;;
	#Modo 2 toma la clave directamente desde la consola ocultando los caracteres
	2)
	stty_orig=`stty -g` #Guardo el estado actual del bash para restaurarlo luego
	echo -n "Ingrese la clave de usuario de su cuenta: "
	stty -echo #Oculto todo los despliegues por pantaña de bash
	read pass
	stty $stty_orig #Restauro el bash
	echo;;
	esac

	#Set mail to alert
	read -p "Ingrese la cuenta email donde enviará la alerta: " alert
}

#########################################
#Crea el saludo al usuario en el correo.#
#########################################
cabecera()
{
hora=`date | awk '{print $4}' | awk -F: '{print $1}'`

if [ $hora -le 5 ]; then
	saludo=`echo "Buenas Noches"`
	else if [ $hora -le 12 ]; then
		saludo=`echo "Buenos Días"`
		else if [ $hora -le 19 ]; then
			saludo=`echo "Buenas Tardes"`
			else
				saludo=`echo "Buenas Noches"`
		fi
	fi
fi
}

#################################################
#Envía la clave, el correo y cierra la conexión.#
#################################################
mail()
{
	sleep 5
	echo $auth
	sleep 2
	echo "mail from: <$umail>"
	sleep 2
	echo "rcpt to: <$alert>"
	sleep 2
	echo data
	sleep 2
	echo "From: MCD <$umail>"
	echo "To: User <$alert>"
	echo "Subject: MCD ALERT!"
	echo
	echo -n $saludo
	echo ", nuestro programa ha detectado un cambio en el directorio ( `echo -n $dir` ) que usted revisa. El detalle es:" 
	cat $dir3
	echo 
	echo .
	sleep 5 
	echo quit
}

#########################################
#Abre la conexión para enviar el correo.#
#########################################
send_mail()
{
	user=`echo $umail | awk -F\@ '{print $1}'` #Obtiene el usuario del correo emisor
	host=`echo $umail | awk -F\@ '{print $2}'` #Obtiene el servidor del correo emisor
	code=`perl -MMIME::Base64 -e 'print encode_base64("\000'$user'\@'$host'\000'$pass'")'`
	auth="AUTH PLAIN $code"
	cabecera
	#Las líneas resultante de la conexion mediante openssl son obviadas, y para
	#mayor comidad, se entregan los dos métodos.
	mail | openssl s_client -starttls smtp -crlf -connect $smtp:$port > /dev/null 2>&1
	#mail | openssl s_client -starttls smtp -crlf -connect $smtp:$port
}

######################
######   MAIN   ######
######################

#Ejecutando función set_mail
set_mail

#Observando el direcotrio ingresado poSe ha detectado un cambio en el
#directorio, se le notificará por correo a $umailr parámetro y guardando su
#estado
ls -l $2 > $dir1

#Observando cambios en el directorio cada $1 segundos
fs1=`stat -c%s "$dir3"`
fs2=$fs1
echo -n "El programa está monitoreando su directorio ( $dir ) ... "
until [ $fs1 -lt $fs2 ]; do
	ls -l $2 > $dir2
	diff $dir1 $dir2 > $dir3
	fs2=`stat -c%s "$dir3"`
	sleep $1
done

echo
echo -n "Se ha detectado un cambio, informando..."

send_mail

#Borrando los elementos temporales usados
rm -f $dir1 $dir2 $dir3 $dir4

echo
echo "Ha finalizado el programa."
